package com.example.demo.department.service;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.department.model.DepartmentEntity;
import com.example.demo.department.repository.DepartmentRepository;

@Service
public class DepartmentService {
    private final DepartmentRepository repository;

    public DepartmentService(DepartmentRepository repository) {
        this.repository = repository;
    }

    public List<DepartmentEntity> getAll() {
        return repository.getAll();
    }

    public DepartmentEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public DepartmentEntity create(DepartmentEntity entity) {
        return repository.create(entity);
    }

    public DepartmentEntity update(Long id, DepartmentEntity entity) {
        final DepartmentEntity existsEntity = get(id);
        existsEntity.setName(entity.getName());
        return repository.update(existsEntity);
    }

    public DepartmentEntity delete(Long id) {
        final DepartmentEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
