package com.example.demo.news.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.core.configuration.Constants;
import com.example.demo.department.service.DepartmentService;
import com.example.demo.news.service.NewsService;
import com.example.demo.news.model.NewsEntity;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/news")
public class NewsController {
    private final NewsService newsService;
    private final ModelMapper modelMapper;
    private final DepartmentService departmentService;

    public NewsController(NewsService newsService, ModelMapper modelMapper, DepartmentService departmentService) {
        this.newsService = newsService;
        this.modelMapper = modelMapper;
        this.departmentService = departmentService;
    }

    private NewsDto toDto(NewsEntity entity) {
        return modelMapper.map(entity, NewsDto.class);
    }

    private NewsEntity toEntity(NewsDto dto) {
        final NewsEntity entity = modelMapper.map(dto, NewsEntity.class);
        entity.setDepartment(departmentService.get(dto.getDepartmentId()));
        return entity;
    }

    @GetMapping
    public List<NewsDto> getAll(@RequestParam(name = "departmentId", defaultValue = "0") Long departmentId) {
        return newsService.getAll(departmentId).stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public NewsDto get(@PathVariable(name = "id") Long id) {
        return toDto(newsService.get(id));
    }

    @PostMapping
    public NewsDto create(@RequestBody @Valid NewsDto dto) {
        return toDto(newsService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public NewsDto update(@PathVariable(name = "id") Long id, @RequestBody NewsDto dto) {
        return toDto(newsService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public NewsDto delete(@PathVariable(name = "id") Long id) {
        return toDto(newsService.delete(id));
    }
}
