package com.example.demo;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.department.model.DepartmentEntity;
import com.example.demo.department.service.DepartmentService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class DepartmentsTests {
    @Autowired
    private DepartmentService departmentService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> departmentService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() {
        departmentService.create(new DepartmentEntity(null, "test1"));
        departmentService.create(new DepartmentEntity(null, "test2"));
        final DepartmentEntity last = departmentService.create(new DepartmentEntity(null, "test3"));
        Assertions.assertEquals(3, departmentService.getAll().size());
        Assertions.assertEquals(last, departmentService.get(3L));
    }

    @Test
    @Order(2)
    void updateTest() {
        final String testName = "TESTNAME";
        final DepartmentEntity entity = departmentService.get(3L);
        final String oldName = entity.getName();
        final DepartmentEntity newEntity = departmentService.update(3L,
                new DepartmentEntity(1L, testName));
        Assertions.assertEquals(3, departmentService.getAll().size());
        Assertions.assertEquals(newEntity, departmentService.get(3L));

        Assertions.assertEquals(testName, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());
    }

    @Test
    @Order(3)
    void deleteTest() {
        departmentService.delete(3L);
        Assertions.assertEquals(2, departmentService.getAll().size());
        final DepartmentEntity last = departmentService.get(2L);
        Assertions.assertEquals(2L, last.getId());

        final DepartmentEntity newEntity = departmentService
                .create(new DepartmentEntity(null, "testtt"));
        Assertions.assertEquals(3, departmentService.getAll().size());
        Assertions.assertEquals(4L, newEntity.getId());
    }
}
