package com.example.demo;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.department.model.DepartmentEntity;
import com.example.demo.directions.model.DirectionsEntity;
import com.example.demo.directions.service.DirectionsService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class DirectionsTests {
    @Autowired
    private DirectionsService directionsService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> directionsService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() {
        directionsService.create(new DirectionsEntity(null, "09.03.01", "Программная инженерия",
                new DepartmentEntity(null, "ИСТ"), "Информатика"));
        directionsService.create(new DirectionsEntity(null, "09.03.02", "ИВТ", new DepartmentEntity(null,
                "ИСТ"),
                "Русский язык"));
        final DirectionsEntity last = directionsService.create(new DirectionsEntity(null, "09.03.03",
                "АиСД", new DepartmentEntity(null, "Гум"), "Математика"));
        Assertions.assertEquals(3, directionsService.getAll().size());
        Assertions.assertEquals(last, directionsService.get(3L));
    }

    @Test
    @Order(2)
    void updateTest() {
        final String testCode = "TESTCODE";
        final String testName = "TESTNAME";
        final DepartmentEntity testDep = new DepartmentEntity(null, "TESTDEP");
        final String testThin = "TESTTHIN";
        final DirectionsEntity entity = directionsService.get(3L);
        final String oldCode = entity.getCode();
        final String oldName = entity.getName();
        final DepartmentEntity oldDep = entity.getDepartment();
        final String oldThin = entity.getThings();
        final DirectionsEntity newEntity = directionsService.update(3L,
                new DirectionsEntity(1L, testCode, testName, testDep, testThin));
        Assertions.assertEquals(3, directionsService.getAll().size());
        Assertions.assertEquals(newEntity, directionsService.get(3L));
        Assertions.assertEquals(testCode, newEntity.getCode());
        Assertions.assertNotEquals(oldCode, newEntity.getCode());

        Assertions.assertEquals(testName, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());

        Assertions.assertEquals(testDep, newEntity.getDepartment());
        Assertions.assertNotEquals(oldDep, newEntity.getDepartment());

        Assertions.assertEquals(testThin, newEntity.getThings());
        Assertions.assertNotEquals(oldThin, newEntity.getThings());
    }

    @Test
    @Order(3)
    void deleteTest() {
        directionsService.delete(3L);
        Assertions.assertEquals(2, directionsService.getAll().size());
        final DirectionsEntity last = directionsService.get(2L);
        Assertions.assertEquals(2L, last.getId());

        final DirectionsEntity newEntity = directionsService
                .create(new DirectionsEntity(null, "test1", "test2",
                        new DepartmentEntity(null, "test3"), "test4"));
        Assertions.assertEquals(3, directionsService.getAll().size());
        Assertions.assertEquals(4L, newEntity.getId());
    }
}
