package com.example.demo;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.department.model.DepartmentEntity;
import com.example.demo.entrysData.model.EntrysDataEntity;
import com.example.demo.entrysData.service.EntrysDataService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class EntrysDataTests {
    @Autowired
    private EntrysDataService entrysDataService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> entrysDataService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() {
        entrysDataService
                .create(new EntrysDataEntity(null, "dyctator", "12345", "user", new DepartmentEntity(null, "ИСТ")));
        entrysDataService.create(new EntrysDataEntity(null, "user2", "12345", "user",
                new DepartmentEntity(null, "ИСТ")));
        final EntrysDataEntity last = entrysDataService.create(new EntrysDataEntity(null, "user3", "12345", "user",
                new DepartmentEntity(null, "Гум")));
        Assertions.assertEquals(3, entrysDataService.getAll().size());
        Assertions.assertEquals(last, entrysDataService.get(3L));
    }

    @Test
    @Order(2)
    void updateTest() {
        final String testLog = "TESTLOG";
        final String testPas = "TESTPAS";
        final DepartmentEntity testDep = new DepartmentEntity(null, "TESTDEP");
        final EntrysDataEntity entity = entrysDataService.get(3L);
        final String oldLog = entity.getLogin();
        final String oldPas = entity.getLogin();
        final DepartmentEntity oldDep = entity.getDepartment();
        final EntrysDataEntity newEntity = entrysDataService.update(3L,
                new EntrysDataEntity(1L, testLog, testPas, entity.getRole(), testDep));
        Assertions.assertEquals(3, entrysDataService.getAll().size());
        Assertions.assertEquals(newEntity, entrysDataService.get(3L));

        Assertions.assertEquals(testLog, newEntity.getLogin());
        Assertions.assertNotEquals(oldLog, newEntity.getLogin());

        Assertions.assertEquals(testPas, newEntity.getPassword());
        Assertions.assertNotEquals(oldPas, newEntity.getPassword());

        Assertions.assertEquals(testDep, newEntity.getDepartment());
        Assertions.assertNotEquals(oldDep, newEntity.getDepartment());
    }

    @Test
    @Order(3)
    void deleteTest() {
        entrysDataService.delete(3L);
        Assertions.assertEquals(2, entrysDataService.getAll().size());
        final EntrysDataEntity last = entrysDataService.get(2L);
        Assertions.assertEquals(2L, last.getId());

        final EntrysDataEntity newEntity = entrysDataService
                .create(new EntrysDataEntity(null, "user4", "12345", "user",
                        new DepartmentEntity(null, "testtt")));
        Assertions.assertEquals(3, entrysDataService.getAll().size());
        Assertions.assertEquals(4L, newEntity.getId());
    }

    @Test
    @Order(4)
    void updatePasswordTest() {
        final String newPas = "TESTPAS";
        final EntrysDataEntity entity = entrysDataService.get(1L);
        final String oldPas = entity.getPassword();
        final EntrysDataEntity newEntity = entrysDataService.updatePassword(1L, oldPas, newPas);

        Assertions.assertEquals(newPas, newEntity.getPassword());
        Assertions.assertNotEquals(oldPas, newEntity.getPassword());
    }
}
